<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

class App
{
    const ENCODING = 'UTF-8';

    private static $instance;
    private static $callbacks;
    private static $plugins;

    private function __construct()
    {
        $this->initPHPConfig();
        $this->initCallbacks();
        $this->initConfig();
        $this->initContainer();
        self::session()->start();
        $this->initView();
        $this->initPlugins();
        $this->initRoute();
        self::runCallbacks('postInit', $this);
    }

    private function initPHPConfig()
    {
        ini_set('session.gc_maxlifetime', 14400);
        mb_language('japanese');
        mb_internal_encoding(self::ENCODING);
        date_default_timezone_set('Asia/Tokyo');
    }

    private function initCallbacks()
    {
        self::$callbacks = array();
    }

    private function initConfig()
    {
        self::register('config', '\App\Config');
    }

    public static function register($name, $class, $params = array(), $callback = null)
    {
        Flight::register($name, $class, $params, $callback);
    }

    private function initContainer()
    {
        self::register('logger', '\App\Logger',
            array(
                self::config()->getLogFile(),
                self::config()->getMaxLogFiles(),
                self::config()->getLogLevel(),
            )
        );
        self::register('session', '\App\Session');
        self::register('item', '\App\Item');
        self::register('itemList', '\App\FileItemList', array(self::config()->itemDataFile()));
        self::register('stockChecker', '\App\StockChecker', array(self::item()));
        // self::register('cart',
        //     function () {
        //         $cart = new \App\Cart(
        //             \App::config()->sameGroupIdOnly(),
        //             \App::config()->useStockManagement(),
        //             \App::config()->getMaxWeight(),
        //             \App::config()->getCartRevision()
        //         );
        //         if (\App::config()->useStockManagement()) {
        //             $cart->addItemChecker(\App::stockChecker());
        //         }
        //         return $cart;
        //     },
        //     array(),
        //     function ($cart) {
        //         $cart->load(\App::session());
        //     });
        self::register('cart',
            '\App\Cart',
            array(
                \App::config()->sameGroupIdOnly(),
                \App::config()->useStockManagement(),
                \App::config()->getMaxWeight(),
                \App::config()->getCartRevision()
            ),
            function ($cart) {
                if (\App::config()->useStockManagement()) {
                    $cart->addItemChecker(\App::stockChecker());
                }
                $cart->load(\App::session());
            });
        self::register('cartItem', '\App\CartItem');
        self::register('validator', '\App\Validator');
        self::register('slip', '\App\Slip', array(), function ($slip) {
            $slip->load(\App::session());
        });
        self::register('stockAllocator', '\App\StockAllocator');
        self::register('mailer', '\App\Mailer');
        self::register('postage', '\App\Postage',
            array(self::config()->getPostageTable(), self::config()->getFreePostagePrice())
        );

        self::register('cartController', '\App\CartController');
        self::register('checkoutController', '\App\CheckoutController');
        self::register('cartService', '\App\Service\Cart');
        self::register('checkoutService', '\App\Service\Checkout');

        self::register('view', 'Smarty', array(), function ($smarty) {
            $smarty->setTemplateDir(\App::config()->templateDir());
            $smarty->setCompileDir(\App::config()->appDir() . '/templates_c/');
            $smarty->setConfigDir(\App::config()->appDir() . '/config/');
            $smarty->setCacheDir(\App::config()->appDir() . '/cache/');
            $smarty->addPluginsDir(\App::config()->appDir() . '/smarty_plugins/');
            $smarty->escape_html = true;
        });
    }

    private function initView()
    {
        self::map('render', function ($template, $data) {
            if (\App::config()->getEnableOutputFilter() && preg_match('/\.html$/', $template)) {
                \App::view()->registerFilter('output', function ($output) use ($template) {
                    $replacement = pack("H*", "2431e38090e8a9a6e794a8e78988e38091");
                    $pattern = pack("H*", "2f283c5c732a7469746c655c732a3e292f69");
                    $out = preg_replace($pattern, $replacement, $output);
                    return $out ? $out : $output;
                });
            }

            $encoding = \App::ENCODING;
            if ($template === '_regi_comp.html') {
                $to_enc = \App::config()->getCompletePageCharset();
                if ($to_enc !== \App::ENCODING) {
                    \App::view()->registerFilter('output', function ($output) use ($to_enc) {
                        $output = mb_convert_encoding($output, $to_enc, \App::ENCODING);
                        return $output;
                    });
                    if (preg_match('/^SJIS/i', $to_enc)) {
                        $encoding = 'Shift_JIS';
                    } else {
                        $encoding = $to_enc;
                    }
                }
                $data['comp_page_charset'] = $to_enc;
            }
            if (!headers_sent()) {
                if($encoding) {
                    header("Content-Type: text/html; charset=$encoding");
                }
                header("X-Frame-Options: DENY");
                header("X-XSS-Protection: 1; mode=block");
                header("X-Content-Type-Options: nosniff");
            }
            $data['comp_page_encoding'] = $encoding;
            \App::view()->assign($data);
            \App::runCallbacks('preRender', $template, $data);
            \App::view()->display($template);
        });

        self::map('fetch', function ($template, $data) {
            if (\App::config()->getEnableOutputFilter() && preg_match('/\.html$/', $template)) {
                \App::view()->registerFilter('output', function ($output) use ($template) {
                    $replacement = pack("H*", "2431e38090e8a9a6e794a8e78988e38091");
                    $pattern = pack("H*", "2f283c5c732a7469746c655c732a3e292f69");
                    $out = preg_replace($pattern, $replacement, $output);
                    return $out ? $out : $output;
                });
            }

            $encoding = \App::ENCODING;
            if ($template === '_regi_comp.html') {
                $to_enc = \App::config()->getCompletePageCharset();
                if ($to_enc !== \App::ENCODING) {
                    \App::view()->registerFilter('output', function ($output) use ($to_enc) {
                        $output = mb_convert_encoding($output, $to_enc, \App::ENCODING);
                        return $output;
                    });
                    if (preg_match('/^SJIS/i', $to_enc)) {
                        $encoding = 'Shift_JIS';
                    } else {
                        $encoding = $to_enc;
                    }
                }
                $data['comp_page_charset'] = $to_enc;
            }
            $data['comp_page_encoding'] = $encoding;
            \App::view()->assign($data);
            \App::runCallbacks('preFetch', $template, $data);
            return \App::view()->fetch($template);
        });

        self::map('error', function ($e) {
            \App::showSystemError($e);
        });
    }

    private function initPlugins()
    {
        if (defined('DISABLE_PLUGIN')) {
            return;
        }
        $config = self::config();
        $disabledPlugins = $config->getDisabledPlugins();
        $pluginsPath = $config->pluginsPath();
        if ($dh = opendir($pluginsPath)) {
            while (($entry = readdir($dh)) !== false) {
                if ($entry == '.' || $entry == '..') {
                    continue;
                }
                if (isset($disabledPlugins[$entry]) && $disabledPlugins[$entry]) {
                    continue;
                }
                $pluginDir = $pluginsPath . '/' . $entry;
                if (!is_dir($pluginDir)) {
                    continue;
                }
                $pluginFile = $pluginDir . '/Plugin.php';
                if (is_readable($pluginFile)) {
                    // self::logger()->debug("Load plugin: $pluginFile");
                    require_once $pluginFile;
                }
            }
            closedir($dh);
        }
    }

    private function initRoute()
    {
        self::map('notFound', function() {
            if(self::isAcceptJson()) {
                self::json([
                    'message' => 'Not Found',

                ], 404);
            }
            else {
                self::_notFound();
            }
        });

        self::runCallbacks('preInitRoute', $this);
        \App\Route::register();
        self::runCallbacks('postInitRoute', $this);
    }

    public static function runCallbacks()
    {
        $num_args = func_num_args();
        if ($num_args < 1) {
            throw new \UnexpectedValueException('コールバック名を指定してください。');
        }

        # NOTE: コールバックの引数は参照渡しにならない 
        $args = func_get_args();
        $name = array_shift($args);
        if (!isset(self::$callbacks[$name])) {
            return false;
        }
        $errors = array();
        $callbacks = self::$callbacks[$name];
        foreach ($callbacks as $priority => $functions) {
            if (!is_array($functions)) {
                continue;
            }
            foreach ($functions as $function) {
                try {
                    if (count($args) == 0) {
                        call_user_func($function);
                    } else {
                        call_user_func_array($function, $args);
                    }
                } catch (Exception $e) {
                    $errors[] = $e->getMessage();
                }
            }
        }
        if (count($errors) > 0) {
            $message = implode("\n", $errors);
            throw new \App\CallbackException($message);
        }
        return true;
    }

    public static function instance()
    {
        if (!self::$instance) {
            self::$instance = new App();
        }
        return self::$instance;
    }

    public static function __callStatic($name, $arguments)
    {
        return Flight::__callStatic($name, $arguments);
    }

    /*
     * index.php(cart.php)でキャッチした例外を表示する。
     * AppやFlightが提供するサービスが使えない状況なのでSmartyを直接生成してエラーテンプレートを表示する。
     * TODO: ログを出力する。
     */

    public static function showSystemError($e)
    {
        error_log("System Error: " . get_class($e) . ": " . $e->getMessage());
        error_log("System Error: " . $e->getTraceAsString());

        $sc_mode = isset($_ENV['SC_MODE']) && $_ENV['SC_MODE'] ? $_ENV['SC_MODE'] : '';
        $cfg = require APP_DIR . '/init' . ($sc_mode ? ".$sc_mode" : '') . '.php';
        $logger = new \App\Logger(
            $cfg['log_file'], $cfg['max_log_files'], $cfg['log_level']
        );
        $logger->error("System Error: " . get_class($e) . ": " . $e->getMessage());
        $logger->debug("System Error: " . $e->getTraceAsString());

        if(self::isAcceptJson()) {
            return self::json([
                'message' => ($sc_mode === 'dev') ? $e->getMessage() : 'Internal Server Error'
            ], 500);
        }

        if (!headers_sent()) {
            header(
                sprintf(
                    "%s %d %s",
                    (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.1'),
                    500,
                    'Internal Server Error'
                ),
                true,
                500
            );
        }

        $smarty = new Smarty();
        $smarty->setTemplateDir(APP_DIR . '/sys_templates/');
        $smarty->setCompileDir(APP_DIR . '/templates_c/');
        $smarty->setConfigDir(APP_DIR . '/config/');
        $smarty->setCacheDir(APP_DIR . '/cache/');
        $smarty->addPluginsDir(APP_DIR . '/smarty_plugins/');
        $smarty->assign(array(
            'error_id' => null,
            'error_message' => ($sc_mode === 'dev') ? $e->getMessage() : '',
            'app_script' => \App::appScriptName(),
            'base_url' => \App::appBasedir(),
        ));
        $smarty->display('_sys_error.html');
    }

    public static function isAcceptJson()
    {
        return (strpos(self::request()->accept, 'application/json') === 0);
    }

    public static function appScriptName()
    {
        $script_name = $_SERVER['SCRIPT_NAME'];
        $script_name = preg_replace('/index.php/', '', $script_name);
        return $script_name;
    }

    public static function appScriptDir()
    {
        $script_name = $_SERVER['SCRIPT_NAME'];
        $app_dir = dirname($script_name);
        return $app_dir;
    }

    public static function appBasedir()
    {
        return \App::request()->base;
    }

    public static function addCallback($name, $callback, $priority = 5)
    {
        if (!isset($name)) {
            return false;
        }
        if (!isset($priority) || $priority < 0 || $priority > 9) {
            return false;
        }
        if (!isset($callback) || !is_callable($callback)) {
            return false;
        }
        if (!isset(self::$callbacks[$name])) {
            self::$callbacks[$name] = array();
        }
        if (!isset(self::$callbacks[$name][$priority])) {
            self::$callbacks[$name][$priority] = array();
        }
        if (array_search($callback, self::$callbacks[$name][$priority]) === false) {
            self::$callbacks[$name][$priority][] = $callback;
            ksort(self::$callbacks[$name]);
        }
        return true;
    }

    public static function addPlugin(\App\Plugin $plugin)
    {
        if (is_null($plugin)) {
            return false;
        }
        self::$plugins[$plugin->key()] = $plugin;
        return $plugin;
    }

    public static function getPlugin($key)
    {
        if(array_key_exists($key, self::$plugins)) {
            return self::$plugins[$key];
        }
        return null;
    }

    public function __destruct()
    {
//		error_log(__METHOD__);
    }

    public function run()
    {
        self::runCallbacks('preRun', $this);
        self::start();
        self::runCallbacks('postRun', $this);
    }

    public static function json($data, $code = 200, $encode = true, $charset = 'utf-8', $option = 0)
    {
        if (!headers_sent()) {
            $http_origin = isset($_SERVER['HTTP_ORIGIN']) ? $_SERVER['HTTP_ORIGIN'] : null;
            $origin = \App::config()->getAllowedOrigin();
            if(isset($http_origin) && isset($origin) && $http_origin === $origin) {
                header("Access-Control-Allow-Origin: $origin");
                header("Access-Control-Allow-Credentials: true");
            }
        }
        Flight::json($data, $code, $encode, $charset, $option);
    }

}


