<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class CheckoutController extends Controller
{
    public function __construct($type = null, $mode = null)
    {
        if (is_null($type)) {
            $type = 'checkout';
        }
        parent::__construct($type, $mode);
    }

    public function _process()
    {
        try {
            $command = $this->getCommand();
            switch ($command) {
                case 'n':
                    $this->show();
                    break;
                case 'nc':
                    $this->confirm();
                    break;
                case 'nccb':
                    $this->order();
                    break;
                case 'cmp':
                    $this->complete();
                    break;
                default:
                    $this->show();
                    break;
            }
        } catch (InvalidCartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            $this->redirect_to_cart();
        } catch (\Exception $e) {
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            $this->render('_error.html', $data);
        }
    }

    public function process()
    {
        try {
            switch ($this->mode) {
                case 'edit':
                    $this->edit();
                    break;
                case 'confirm':
                    $this->confirm();
                    break;
                case 'order':
                    $this->order();
                    break;
                case 'complete':
                    $this->complete();
                    break;
                default:
                    $this->show();
                    break;
            }
        } catch (InvalidCartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $this->redirect_to_cart();
        } catch (\Exception $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            $this->render('_error.html', $data);
        }
    }

    public function getTemplateParams()
    {
        $data = parent::getTemplateParams();
        $slip = \App::slip();
        $slip_data = $slip->getProperties();
        $data = array_merge($data, $slip_data);
        $labels = \App::config()->slipOptionLabels();
        $data = array_merge($data, $labels);
        return $data;
    }

    public function getCompatibleParams()
    {
        $request = \App::request();
        $params = array();
        foreach ($request->data as $k => $v) {
            if (preg_match('/^slip_/', $k)) {
                $params[mb_substr($k, 5)] = $v;
            }
        }
        return $params;
    }

    public function getConfirmParams()
    {
        $request = \App::request();
        $params = array();
        foreach ($request->data as $k => $v) {
            if (preg_match('/^(?:user_|payment_|delivery_|memo)/', $k)) {
                $params[$k] = $v;
            }
        }
        return $params;
    }

    public function modifyTemplateParams($data)
    {
        $removeHyphen = \App::config()->getRemoveTelHyphenAtRegComplete();
        if ($removeHyphen) {
            $data['user_tel'] = str_replace('-', '', $data['user_tel']);
            $data['user_fax'] = str_replace('-', '', $data['user_fax']);
        }
        return $data;
    }

    public function _redirectToCompletePage()
    {
        $config = \App::config();
        $scriptName = \App::appScriptName();
        if (preg_match('/\.php$/', $scriptName)) {
            $scriptName = basename($scriptName);
        } else {
            $scriptName = '';
        }
        $url = $scriptName . "?type=checkout&mode=complete";
        $completePageURL = $config->getCompletePageURL();
        if (!empty($completePageURL)) {
            if (!preg_match('/\/$/', $completePageURL)) {
                $completePageURL .= '/';
            }
            $url = $completePageURL . $url;
        }
        $this->redirect($url);
    }

    public function redirectToCompletePage()
    {
        $config = \App::config();
        $url = \App\Util::make_url('checkout', 'complete', false);
        $completePageURL = $config->getCompletePageURL();
        if (!empty($completePageURL)) {
            $url = $completePageURL;
        }
        $this->redirect($url);
    }

    public function show()
    {
        $this->edit();
    }

    public function edit()
    {
        $service = \App::checkoutService();
        $service->createEmptySlip();
        $data = $this->getTemplateParams();
        $this->render('_regi.html', $data);
    }

    public function confirm()
    {
        $errors = null;
        $params = $this->getConfirmParams();
        $service = \App::checkoutService();
        try {
            $service->createSlip($params);
        } catch (ValidationException $e) {
            $errors = $e->getErrors();
        }

        $templateFile = '_regi_conf.html';
        $data = $this->getTemplateParams();
        if (isset($errors)) {
            $data = $this->addValidationErrorMessage($data, $errors);
            $data = $this->addInputParams($data);
            $templateFile = '_regi.html';
        }
        $this->render($templateFile, $data);
    }

    public function order()
    {
        $service = \App::checkoutService();
        $service->order();
        $this->redirectToCompletePage();
    }

    public function complete()
    {
        $service = \App::checkoutService();
        try {
            $service->complete();
        } catch (CartException $e) {
            $this->redirect_to_cart();
        } catch (\Exception $e) {
            throw $e;
        }

        $data = $this->getTemplateParams();
        $data = $this->modifyTemplateParams($data);
        $this->render('_regi_comp.html', $data);
    }

}


