<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class Config
{
    const MAX_NUM_OPTIONS = 30;
    const MAX_NUM_SLIP_OPTIONS = 10;
    const ENABLE_OUTPUT_FILTER = true;
    const PREVIEW_TEMPLATE = false;

    protected $cfg;
    protected $APP_DIR;

    public function __construct()
    {
        if (PHP_SAPI === 'cli') {
            $this->APP_DIR = isset($_ENV['APP_DIR']) ? $_ENV['APP_DIR'] : __DIR__ . '/../..';
        } else {
            $this->APP_DIR = APP_DIR;
        }

        $config_file = $this->APP_DIR . '/init.php';
        $sc_mode = getenv('SC_MODE');
        if (isset($sc_mode) && preg_match('/^(?:dev|test)$/', $sc_mode)) {
            $_config_file = $this->APP_DIR . "/init.$sc_mode.php";
            if (is_readable($_config_file)) {
                $config_file = $_config_file;
            }
        }
        $this->cfg = require $config_file;
    }

    public function __get($key)
    {
        return $this->get($key);
    }

    public function get($key, $default = null)
    {
        if (array_key_exists($key, $this->cfg)) {
            return $this->cfg[$key];
        }
        return $default;
    }

    public function set($key, $value)
    {
        $this->cfg[$key] = $value;
    }

    public function itemDataFile()
    {
        return $this->APP_DIR . '/' . $this->item_filename;
    }

    public function pluginsPath()
    {
        return $this->APP_DIR . '/plugins';
    }

    public function cartItemOptionLabel($n)
    {
        $key = "cart_item_opt{$n}_label";
        return $this->get($key);
    }

    public function cartItemOptionLabels()
    {
        $labels = array();
        foreach (range(1, static::MAX_NUM_OPTIONS) as $n) {
            $key = "cart_item_opt{$n}_label";
            if (array_key_exists($key, $this->cfg)) {
                $labels[$key] = $this->cfg[$key];
            }
        }
        return $labels;
    }

    public function slipOptionLabels()
    {
        $labels = array();
        foreach (range(1, static::MAX_NUM_SLIP_OPTIONS) as $n) {
            foreach (array('user', 'delivery') as $type) {
                $key = "slip_{$type}_opt{$n}_label";
                $labels[substr($key, 5)] = $this->get($key, '');
            }
        }
        return $labels;
    }

    public function templateDir()
    {
        $path = $this->get('template_dir', 'templates');
        $path = $this->appDir() . '/' . $path . '/';
        return $path;
    }

    public function appDir()
    {
        return $this->APP_DIR;
    }

    public function maxNumOptions()
    {
        return self::MAX_NUM_OPTIONS;
    }

    public function inputCheck($name)
    {
        $keys = array(
            'input_check',
            'input_check_user_option',
            'input_check_delivery',
            'input_check_delivery_option',
        );
        foreach ($keys as $key) {
            $inputCheck = $this->get($key);
            if (isset($inputCheck)) {
                if (array_key_exists($name, $inputCheck)) {
                    $value = explode(',', $inputCheck[$name]);
                    return $value;
                }
            }
        }
        return false;
    }

    public function getPaymentWayName($index)
    {
        $paymentWay = $this->getPaymentWay($index);
        $paymentWayArray = explode(',', $paymentWay);
        if (!isset($paymentWayArray[0]) || empty($paymentWayArray[0])) {
            throw new ConfigException('決済方法の名称が設定されていません。');
        }
        $name = $paymentWayArray[0];
        return $name;
    }

    public function getPaymentWay($index)
    {
        $paymentWays = $this->get('payment_way');
        if (!isset($paymentWays)) {
            throw new ConfigException('決済方法が設定されていません。');
        }
        // 0 は無効
        if ($index <= 0 || $index >= count($paymentWays)) {
            throw new ConfigException('無効な決済方法です。');
        }
        return $paymentWays[$index];
    }

    public function getPaymentWayFee($index)
    {
        $paymentWay = $this->getPaymentWay($index);
        $paymentWayArray = explode(',', $paymentWay);
        if (!isset($paymentWayArray[1])) {
            throw new ConfigException('決済方法の手数料が設定されていません。');
        }
        $fee = intval($paymentWayArray[1]);
        return $fee;
    }

    public function getPaymentWayType($index)
    {
        $paymentWay = $this->getPaymentWay($index);
        $paymentWayArray = explode(',', $paymentWay);
        if (!isset($paymentWayArray[2])) {
            throw new ConfigException('決済方法の種別が設定されていません。');
        }
        $type = intval($paymentWayArray[2]);
        return $type;
    }

    public function getDefaultPaymentWay()
    {
        return $this->get('default_payment_way', null);
    }

    public function getTaxRate()
    {
        $rate = $this->get('tax_rate');
        if (is_null($rate)) {
            throw new ConfigException('消費税率が設定されていません。');
        }
        return $rate * 0.01;
    }

    public function getPostage($prefId)
    {
        $postages = $this->getPostageTable();
        if ($prefId < 1 || $prefId > 47) {
            throw  new ConfigException('無効な都道府県コードです。');
        }
        return $postages[$prefId];
    }

    public function getPostageTable()
    {
        $postages = $this->get('pref_postage');
        if (!is_array($postages)) {
            throw new ConfigException('送料が設定されていません。');
        }
        return $postages;
    }

    public function getFreePostagePrice()
    {
        $price = $this->get('free_delivery_price');
        if (is_null($price)) {
            throw new ConfigException('送料無料価格が設定されていません。');
        }
        if ($price === '') {
            $price = false;
        }
        return $price;
    }

    public function useStockManagement()
    {
        return $this->get('item_amount_use_sw') ? true : false;
    }

    public function getMaxWeight()
    {
        return intval($this->get('item_weight_max'));
    }

    public function sameGroupIdOnly()
    {
        return $this->get('same_group_id_only', 1) ? true : false;
    }

    public function getMailSettings()
    {
        $settings = array(
            array(
                'to' => $this->get('email_admin_to'),
                'toColumn' => $this->get('email_admin_to_user') ? 'user_mail' : null,
                'from' => $this->get('email_admin_from'),
                'replyTo' => $this->get('email_admin_reply'),
                'replyToColumn' => 'user_mail',
                'subject' => $this->get('email_admin_subject'),
                'replyToUser' => $this->get('email_admin_reply_to_user'),
                'includeOrderId' => $this->get('email_subject_include_order_id'),
                'template' => '_mail_order.txt',
            ),
        );
        if ($this->get('email_send_copy_to_user')) {
            $settings[] = array(
                'toColumn' => 'user_mail',
                'from' => $this->get('email_user_from'),
                'replyTo' => $this->get('email_user_reply'),
                'subject' => $this->get('email_user_subject'),
                'includeOrderId' => $this->get('email_subject_include_order_id'),
                'template' => '_mail_user.txt',
            );
        }
        if ($this->getEnableOutputFilter()) {
            for ($i = 0; $i < count($settings); $i++) {
                if (array_key_exists('subject', $settings[$i])) {
                    $prefix = pack("H*", "e38090534b454c45544f4e204341525420e8a9a6e794a8e78988e38091");
                    $settings[$i]['subject'] = $prefix . $settings[$i]['subject'];
                }
            }
        }
        return $settings;
    }

    public function getCompletePageURL()
    {
        return $this->get('comp_page_url');
    }

    public function getCompletePageCharset()
    {
        return $this->get('comp_page_charset');
    }

    public function getDisabledPlugins()
    {
        return $this->get('disabled_plugins', array());
    }

    public function getLogFile()
    {
        $logFile = $this->get('log_file');
        if ($logFile) {
            $logFile = $this->appDir() . '/' . $logFile;
        }
        return $logFile;
    }

    public function getMaxLogFiles()
    {
        return $this->get('max_log_files', 14);
    }

    public function getLogLevel()
    {
        $levelStr = $this->get('log_level', 'INFO');
        $levelStr = '\Monolog\Logger::' . strtoupper($levelStr);
        $level = constant($levelStr);
        if (is_null($level)) {
            $level = \Monolog\Logger::INFO;
        }
        return $level;
    }

    public function getRemoveTelHyphenAtRegComplete()
    {
        return $this->get('comp_page_use_tel_hyphen');
    }

    public function getMaintenanceMode()
    {
        return $this->get('maintenance_mode') ? true : false;
    }

    protected function getMaintenanceTime($time)
    {
        if (\App\Util::is_empty($time)) {
            \App::logger()->error('メンテナンスモード日時が空です。');
            return false;
        }
        $epoch = strtotime($time);
        if ($epoch === false) {
            \App::logger()->error('不正なメンテナンスモード日時です。(' . $time . ')');
            return false;
        }
        return $epoch;
    }

    public function getMaintenanceTimeStart()
    {
        $time = $this->get('maintenance_time_start', '');
        return $this->getMaintenanceTime($time);
    }

    public function getMaintenanceTimeEnd()
    {
        $time = $this->get('maintenance_time_end', '');
        return $this->getMaintenanceTime($time);
    }

    public function getCartRevision()
    {
        return $this->get('cart_revision', 1);
    }

    public function getLogOrder()
    {
        return $this->get('log_order') ? true : false;
    }

    public function getEnableOutputFilter()
    {
        return self::ENABLE_OUTPUT_FILTER;
    }

    public function isPreviewMode()
    {
        return self::PREVIEW_TEMPLATE;
    }

    public function getAllowedOrigin()
    {
        return $this->get('allowed_origin', '');
    }

    public function getCartPath()
    {
        return $this->get('cart_path', '/cart');
    }

    public function getCheckoutPath()
    {
        return $this->get('checkout_path', '/checkout');
    }
}


